% function [cent, Memb]=fcm_var_q2(filename,c,exponent,beta,thresh,verbose,maxiter)
% 
% Fuzzy C means with variable compactness and smoothness factor beta for 3 
% classes. 
% c : initial centroid (3x1 vector)
% exponent : [p,q,r] are exponents of 3 classes. Compared to usual FCM, 
% p=1/(q-1), q>1. Use exponent = [1 1 1] to get usual q=2 based FCM
% beta : smoothness parameter, usually in the range [0.0,0.2]
% Thresh : image threshold (usually 0)
% maxiter : maximum number of iterations (usually 20)
% All fuzzy membership volumes are given in the structure Memb. 
% Filename is the full location of the file, either NIFTI or XML. 
% It can also be a 3D matrix.
 

function [cent, Memb]=fcm_var_q2(filename,c,exponent,beta,thresh,verbose,maxiter)

if nargin<7
    maxiter=20;
end
try 
    x=verbose;
catch e
    verbose=0;
end
N=length(c);
if length(exponent)==1
    exponent=exponent*ones([N 1]);
end

try
if ischar(filename)
    if strcmpi(filename(end-2:end),'xml')
        [Vol, pX]=ReadXml(filename);
%         dim=pX.dim;
    elseif strcmpi(filename(end-2:end),'nii')
        temp=load_untouch_nii(filename);
        Vol=double(temp.img);
%         dim=size(Vol);
    elseif isnumeric(filename)
        Vol=filename;
    end
else
    Vol=filename;
end
catch e
    fprintf('Unknown file type, only xml and nii are supported\n');
    fprintf('Check file name and type\n Program will exit now\n');
    cent=[];
    Memb=[];
    fprintf('%s\n',e.message);
    return;
end

fprintf('Normalizing by WM peak. ');
[I, ~ ,~ ,~]=OutlierReduction(Vol,0,0,[0 2]);
temp=Vol;
temp(temp>I(2))=0;
temp(temp<I(1) & temp>0)=0;
pS=find_peaks(temp,[],[],0,2,(I(2)-I(1))/80);
Vol=Vol/pS(end);
if verbose
fprintf('WM peak found at %.4f\n',pS(end));
end
c=c/pS(end);
if verbose
    fprintf('New initial centroids = ');
    for i=1:N
    fprintf('%.4f, ',c(i));
    end
    fprintf('\n');
end


mask=Vol>thresh;

dim=size(Vol);
err=max(Vol(:));
count=0;
dfilt(:,:,1)=[0 0 0
    0 1 0
    0 0 0];
dfilt(:,:,2)=[0 1 0
    1 0 1
    0 1 0];
dfilt(:,:,3)=[0 0 0
    0 1 0
    0 0 0];
dfilt=dfilt/6;
u=cell(N,1);
for i=1:N
    u{i,1}=zeros(dim);
end
epsm=1E-20;
E=ones(dim)*epsm;
temp=cell(N,1);
B=cell(N,1);
c_new=zeros(1,length(exponent));
th=max(Vol(:))/1000;
if verbose
fprintf('Iteration error threshold set at %.3f.\n',th);
end
while err>th && count<maxiter
        
    for i=1:N
        B{i,1}=convn(u{i,1}.^2,dfilt,'same');
    end
    C=zeros(dim);
    for i=1:N
        C=C+B{i};
    end
    for i=1:N
        temp{i,1}=zeros(dim);
    end
    
    for i=1:N
        temp{i,1}=((Vol-c(i)).^2).^exponent(i);
        
        temp{i,1}=temp{i,1}+beta*(C - B{i} );
        
    end
            
    for i=1:N
        u{i,1}=1./temp{i,1};
    end
    S=zeros(dim);
    for i=1:N
        S=S+u{i,1};
    end
    for i=1:N
        u{i,1}=mask.*(u{i,1}./S);
    end
    clear S;
    for i=1:N
        u{i,1}(isnan(u{i,1})==1)=0;
    end
    for i=1:N
        p=exponent(i);
        c_new(i)=sum(sum(sum(mask.*Vol.*(u{i,1}.^2).*((E+(Vol-c(i)).^2).^(p-1)))))/...
            sum(sum(sum(mask.*(u{i,1}.^2).*((epsm+(Vol-c(i)).^2).^(p-1)))));
    end
      
    
    err=max(abs(c-c_new));
    if verbose
        fprintf('Iteration=%d, error= %.5f, centroid=',count,err);
        for i=1:N
            fprintf('%.3f ',c(i));
        end
        fprintf('\n');
    end
    
    c=sort(c_new,'ascend');
    count=count+1;
end
if ~verbose
    fprintf('Iteration=%d, error= %.5f, centroid=',count,err);
    for i=1:N
        fprintf('%.3f ',c(i));
    end
    fprintf('\n');
end
cent=c_new;
Memb=cell(N,1);
for i=1:N
    Memb{i,1}=u{i,1}.*mask;
end
       


